@@ -13,6 +13,7 @@
 #include <QCryptographicHash>
 #include <QFile>
 #include <QIODevice>
+#include <QRandomGenerator>
 #include <QSaveFile>
 #include <assert.h>
 #ifdef HAVE_GPGMEPP
@@ -28,12 +29,6 @@
 #include "kwalletbackend.h"
 #include "sha1.h"
 
-#ifdef Q_OS_WIN
-#include <windows.h> // Must be included before wincrypt.h
-
-#include <wincrypt.h>
-#endif
-
 #define KWALLET_CIPHER_BLOWFISH_ECB 0 // this was the old KWALLET_CIPHER_BLOWFISH_CBC
 #define KWALLET_CIPHER_3DES_CBC 1 // unsupported
 #define KWALLET_CIPHER_GPG 2
@@ -47,90 +42,6 @@ namespace KWallet
 {
 typedef char Digest[16];
 
-static int getRandomBlock(QByteArray &randBlock)
-{
-#ifdef Q_OS_WIN // krazy:exclude=cpp
-
-    // Use windows crypto API to get randomness on win32
-    // HACK: this should be done using qca
-    HCRYPTPROV hProv;
-
-    if (!CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT | CRYPT_SILENT)) {
-        return -1; // couldn't get random data
-    }
-
-    if (!CryptGenRandom(hProv, static_cast<DWORD>(randBlock.size()), (BYTE *)randBlock.data())) {
-        return -3; // read error
-    }
-
-    // release the crypto context
-    CryptReleaseContext(hProv, 0);
-
-    return randBlock.size();
-
-#else
-
-    // First try /dev/urandom
-    if (QFile::exists(QStringLiteral("/dev/urandom"))) {
-        QFile devrand(QStringLiteral("/dev/urandom"));
-        if (devrand.open(QIODevice::ReadOnly)) {
-            int rc = devrand.read(randBlock.data(), randBlock.size());
-
-            if (rc != randBlock.size()) {
-                return -3; // not enough data read
-            }
-
-            return 0;
-        }
-    }
-
-    // If that failed, try /dev/random
-    // FIXME: open in noblocking mode!
-    if (QFile::exists(QStringLiteral("/dev/random"))) {
-        QFile devrand(QStringLiteral("/dev/random"));
-        if (devrand.open(QIODevice::ReadOnly)) {
-            int rc = 0;
-            int cnt = 0;
-
-            do {
-                int rc2 = devrand.read(randBlock.data() + rc, randBlock.size());
-
-                if (rc2 < 0) {
-                    return -3; // read error
-                }
-
-                rc += rc2;
-                cnt++;
-                if (cnt > randBlock.size()) {
-                    return -4; // reading forever?!
-                }
-            } while (rc < randBlock.size());
-
-            return 0;
-        }
-    }
-
-    // EGD method
-    QString randFilename = QString::fromLocal8Bit(qgetenv("RANDFILE"));
-    if (!randFilename.isEmpty()) {
-        if (QFile::exists(randFilename)) {
-            QFile devrand(randFilename);
-            if (devrand.open(QIODevice::ReadOnly)) {
-                int rc = devrand.read(randBlock.data(), randBlock.size());
-                if (rc != randBlock.size()) {
-                    return -3; // not enough data read
-                }
-                return 0;
-            }
-        }
-    }
-
-    // Couldn't get any random data!!
-    return -1;
-
-#endif
-}
-
 BackendPersistHandler *BackendPersistHandler::getPersistHandler(BackendCipherType cipherType)
 {
     switch (cipherType) {
@@ -245,12 +156,7 @@ int BlowfishPersistHandler::write(Backend *wb, QSaveFile &sf, QByteArray &versio
 
     QByteArray randBlock;
     randBlock.resize(blksz + delta);
-    if (getRandomBlock(randBlock) < 0) {
-        sha.reset();
-        decrypted.fill(0);
-        sf.cancelWriting();
-        return -3; // Fatal error: can't get random
-    }
+    QRandomGenerator::securelySeeded().generate(randBlock.begin(), randBlock.end());
 
     for (int i = 0; i < blksz; i++) {
         wholeFile[i] = randBlock[i];
@@ -564,7 +470,7 @@ int GpgPersistHandler::write(Backend *wb, QSaveFile &sf, QByteArray &version, WI
     }
 
     char buffer[4096];
-    ssize_t bytes = 0;
+    qint64 bytes = 0;
     encryptedData.seek(0, SEEK_SET);
     while ((bytes = encryptedData.read(buffer, sizeof(buffer) / sizeof(buffer[0]))) > 0) {
         if (sf.write(buffer, bytes) != bytes) {
@@ -604,7 +510,7 @@ int GpgPersistHandler::read(Backend *wb, QFile &sf, WId w)
     // the remainder of the file is GPG encrypted. Let's decrypt it
     GpgME::Data encryptedData;
     char buffer[4096];
-    ssize_t bytes = 0;
+    qint64 bytes = 0;
     while ((bytes = sf.read(buffer, sizeof(buffer) / sizeof(buffer[0])))) {
         encryptedData.write(buffer, bytes);
     }
